<?php
/**
	PhpCustomTagEngine.php
	
	Written by Ryu, Gwang. 
	gryu@korea.com, http://www.gpgstudy.com/
*/

define ("PCT_TEMPLATE", 0);
define ("PCT_HTML", 1);

class PhpCustomTag
{
	var $Name;
	var $Attributes;
	var $Content;
	var $Result;
	var $Type;
	function PhpCustomTag($aDef, $aContent, $aType=PCT_TEMPLATE  )
	{
		$this->Content = $aContent;
		$this->Type = $aType;
		$this->Name = $aDef["Name"];
		$this->Attributes = $aDef["Attributes"];
	}
	
	function expand($dataArray)
	{
		$this->Result = $this->Content;
		foreach ($dataArray as $key => $value) 
		{
			$this->Result = str_replace('{'.$key.'}', $value, $this->Result) ;
			
		}
	}
}

class PhpCustomTagParser
{
	var $TagCollection = array();
	var $TemplateData;
	var $Settings;
	var $StartPrefix;
	var $StartPrefixLenght;
	var $EndPrefix;
	var $EndPrefixLenght;
	var $Params = array();
	
	function PhpCustomTagParser($aFilename = '',  $Prepare = false )
	{
		$this->setPrefix('pct');
		if ($Prepare)
		{
			$this->loadFromFile($aFilename);
			$this->prepare();
		}
	}
	
	function setTemplate($aTemplateData)
	{
		$this->TemplateData = $aTemplateData;
	}
	function setParam($key, $val)
	{
		$this->Params[$key] = $val;
	}
		
	function setPrefix($aPrefix)
	{
		$this->Settings['PREFIX'] = $aPrefix;
		$this->StartPrefix = '<' . $aPrefix . ':';
		$this->EndPrefix = '</' . $aPrefix . ':';
		
		$this->StartPrefixLength = strlen($this->StartPrefix);
		$this->EndPrefixLength = strlen($this->EndPrefix);

	}
	
	function loadFromFile($aFilename)
	{
		 $this->setTemplate( implode("\r\n", file($aFilename) ) );
	}
	
	function prepare()
	{
		
		$StartPos = 0;
		$Data = $this->TemplateData;
		$EndPos = 0;
		$HtmlSource = $Data;
		$done = false;

		while ( !$done )
		{
			$StartPos = strpos($Data, $this->StartPrefix, $EndPos);
			
			if ($StartPos === false) {
				$HtmlSource = substr($Data, $EndPos);
				$this->TagCollection[] = new PhpCustomTag('HTML',  $HtmlSource, PCT_HTML);
				$done = true;
			} 
			else 
			{
				$HtmlSource = trim( substr($Data, $EndPos, $StartPos - $EndPos) );
				if ( strlen( $HtmlSource ) > 0) {
					$this->TagCollection[] = new PhpCustomTag('HTML', $HtmlSource, PCT_HTML);
				}
				$StartPos += $this->StartPrefixLength;  // '<rt:' 
				$EndPos = strpos($Data, '>', $StartPos);
				$TagDef = $this->parseTag( trim( substr($Data, $StartPos, $EndPos - $StartPos) ) );
				$TagName = trim( $TagDef["Name"] );
				$StartPos = strpos($Data, $this->EndPrefix.$TagName.'>' , $EndPos);

				if ($StartPos === false ) {
					$done = true;
				}
				else
				{
					$TagContent = trim(substr($Data, $EndPos + 1, $StartPos - $EndPos - 1));
					$this->TagCollection[] = new PhpCustomTag($TagDef, $TagContent, PCT_TEMPLATE);
				
					$EndPos = $StartPos + $this->EndPrefixLength + strlen($TagName) + 1;
				}
			}
		}
	}
	function parse()
	{
		$result = '';
		 $Prefix  = $this->Settings['PREFIX'];
		foreach ($this->TagCollection as $tag)
		{
			switch ($tag->Type ) {
				case PCT_HTML :
					$result .= $tag->Content;
					break;
				case PCT_TEMPLATE:
					$HandlerClassName =  $Prefix . $tag->Name;
					$TagHandler = new $HandlerClassName( $this->Params );
					do 
					{
						$CallAgain = false;
						$result .= $TagHandler->getResult($tag, &$CallAgain);
					} while ($CallAgain);
					break;
					
			}
		}
		return $result;
	}	
	function parseTag($TagSrc)
	{
		$result = array();
		$temp = str_replace( '"', '', $TagSrc) ;
		$temp = str_replace("'", '', $temp);
	
		$tok = strtok($temp, ' ');
		$result["Name"] = $tok;
		$key = trim( strtok( '=' ) );
		$attributes = array();
		while ($key)
		{
			$val = trim( strtok( ' ' ) );
			$attributes[$key] = $val;
			$key = trim( strtok( '=' ) );
		}
		$result["Attributes"] = $attributes;
		return $result;
	}

}

		